<?php
class CacheBuilder {

    private $mcd;
    private $logger;
    private $pdo;
    private $templ;

    public function __construct($logger, $pdo, $templ) {
        $this->logger = $logger;
        $this->pdo = $pdo;
        $this->templ = $templ;
    }

    private function __clone() { }

    public function retrieve_page($path) {
        if (file_exists($path)) {
            header("Location: $path");
        } else {
            $path2 = str_replace("/", "-", $path);
            $path2 = str_replace(".", "-", $path2);
            $cache_key = "page-$path2";
            if (!($html = apcu_fetch($cache_key))) {
                return $this->detect_and_build($path);
            } else {
                return $html;
            }
        }
    }

    public function detect_and_build($path) {
        if (!($type = $this->detect_url($path))) {
            $this->logger->log_error("PAGE_BUILDER", "The path $path is of unknown type.");
            return false;
        }
        $html = "";
        if ($type == "board_index") {
            // index = /$board/
            // index pages = /$board/$page
            $split = explode("/", $path);
            $board = $split[1];
            $page = $split[2];

            $info_query = $this->pdo->prepare("SELECT * FROM boards WHERE board_name = ?")->execute([$board]);
            $info = $info_query->fetch(PDO::FETCH_ASSOC);

            $html = $this->templ->fill_template("board_template", array("thread-name" => "", "board-name" => $board, "chan-name" => $config["CHAN_NAME"], "banner-url" => "TODO banners!", "board-title" => $info["title"], "board-description" => $info["description"], "board-news" => $info["news"], "copyright-info" => "<a href='https://github.com/mission712/blazechan'>Blazechan</a> by mission712.<br />".$config["COPY-INFO"]));
            $threads_query = $this->pdo->prepare("SELECT * FROM posts WHERE board = ? AND is_op = 1 ORDER BY timestamp DESC LIMIT 15 OFFSET ?")->execute([$board, $page ? $page * 15 : 0]);
            $threads = "";
            foreach ($threads_query->fetchAll(PDO::FETCH_ASSOC) as $thread) {
                $threads .= $this->templ->fill_template("post_op", array("subject" => $thread["subject"], "name" => $thread["name"], "tripcode" => $thread["tripcode"], "tstamp" => $thread["timestamp"], "time" => date("%Y-%m-%d %H:%M:%S", $thread["timestamp"]), "thread" => $thread["post_id"], "media-link" => $thread["media-link"], "post-thumb" => $thread["post-thumb"]));
                // TODO omits
                // TODO multiple images
                $posts_query = $this->pdo->prepare("SELECT * FROM posts WHERE board = ? AND on_thread = ? ORDER BY timestamp ASC LIMIT 5 OFFSET ?")->execute([$board, $thread["post_id"], $thread["posts"] - 5]);
                $posts = "";
                foreach ($posts_query->fetchAll(PDO::FETCH_ASSOC) as $post) {
                    $posts .= $this->templ->fill_template("post_normal", array("subject" => $post["subject"], "name" => $post["name"], "tripcode" => $post["tripcode"], "tstamp" => $post["timestamp"], "time" => date("%Y-%m-%d %H:%M:%S", $post["timestamp"]), "thread" => $thread["post_id"], "post" => $post["post_id"], "media-link" => $post["media-link"], "post-thumb" => $post["post-thumb"]));
                }
                $threads .= $posts;
            }
            // append page content
            $html = str_replace("{{page-content}}", $threads, $html);
            
            apcu_store("page-".str_replace("/", "-", $path), $html, 300);
            return $html;
        } elseif ($type == "thread") {
            // /$board/thread/$thread
            $split = explode("/", $path);
            $board = $split[1];
            $thread_id = $split[3];

            $info_query = $this->pdo->prepare("SELECT * FROM boards WHERE board_name = ?")->execute([$board]);
            $info = $info_query->fetch(PDO::FETCH_ASSOC);

            $html = $this->templ->fill_template("board_template", array("board-name" => $board, "chan-name" => $config["CHAN_NAME"], "banner-url" => "TODO banners!", "board-title" => $info["title"], "board-description" => $info["description"], "board-news" => $info["news"], "copyright-info" => "<a href='https://github.com/mission712/blazechan'>Blazechan</a> by mission712.<br />".$config["COPY-INFO"]));

            $threads_query = $this->pdo->prepare("SELECT * FROM posts WHERE board = ? AND is_op = 1 AND post_id = ?")->execute([$board, $THREAD]);
            $thread = $this->templ->fill_template("post_op", array("subject" => $thread["subject"], "name" => $thread["name"], "tripcode" => $thread["tripcode"], "tstamp" => $thread["timestamp"], "time" => date("%Y-%m-%d %H:%M:%S", $thread["timestamp"]), "thread" => $thread["post_id"], "media-link" => $thread["media-link"], "post-thumb" => $thread["post-thumb"], "body" => $thread["body"]));
            // TODO multiple images
            
            // replace thread title
            $html = str_replace("{{thread-title}}", ($thread["subject"] || substr($thread["body"], 0, strpos($thread["body"], "<br />")))." - ", $html);

            $posts_query = $this->pdo->prepare("SELECT * FROM posts WHERE board = ? AND on_thread = ? ORDER BY timestamp ASC")->execute([$board, $thread["post_id"]]);
            $posts = "";
            foreach ($posts_query->fetchAll(PDO::FETCH_ASSOC) as $post) {
                $posts .= $this->templ->fill_template("post_normal", array("subject" => $post["subject"], "name" => $post["name"], "tripcode" => $post["tripcode"], "tstamp" => $post["timestamp"], "time" => date("%Y-%m-%d %H:%M:%S", $post["timestamp"]), "thread" => $thread["post_id"], "post" => $post["post_id"], "media-link" => $post["media-link"], "post-thumb" => $post["post-thumb"]));
            }
            $threads .= $posts;

            // append page content
            $html = str_replace("{{page-content}}", $threads, $html);
            
            apcu_store("page-".str_replace("/", "-", $path), $html, 2400);
            return $html;
        }
        return false;
    }

    public function detect_url($path) {
        $arr = explode("/", explode("#", $path)[0]);
        if (count($arr) > 2) {
            if ($arr[2] == "") {
                return "board_index";
            }

            try {
                $arr[2] = (int) $arr[2];
                return "board_index";
            } catch (Exception $e) {
                if ($arr[2] == "thread") {
                    try {
                        $arr[3] = (int) $arr[3];
                        return "thread";
                    } catch (Exception $e) {
                        return false;
                    }
                }
            }
        }
        return $this->detect_url($path."/");
    }
}
?>
