<?php
    class TemplateBuilder {
            protected $pages = array();
            protected $logger;

            public function __construct($logger) {
                    $this->logger = $logger;
                    foreach (new APCIterator("user") as $page) {
                            if (substr($page['key'], 0, 9) == "template_") {
                                    $this->pages[substr($page['key'], 8)] = $page['value'];
                            }
                    }
            }

            private function __clone() {

            }

            public function store_existing($path, $name) {
                    if (apc_exists($name)) {
                            return;
                    }
                    $file = fopen($path, "rb");
                    if (!$file) {
                            $this->logger->log_error("TEMPLATE_PAGE", "Failed to open file $path to store page");
                            return false;
                    }
                    $this->pages[$name] = $path;
                    if (apc_store("template_".$name, fread($file, 2**16), 1200)) {
                        $this->logger->log_info("TEMPLATE_PAGE", "Successfully stored file $name");
                    }
            }

            public function store_raw_html($text, $name) {
                    if (apc_exists($name)) {
                            return;
                    }
                    $this->pages[$name] = $text;
                    if (apc_store("template_".$name, $text, 1200)) {
                        $this->logger->log_info("TEMPLATE_PAGE", "Successfully stored HTML data for $name");
                    }
            }

            public function retrieve_page($name) {
                    if(!array_key_exists($name, $this->pages)) {
                            $this->logger->log_error("TEMPLATE_PAGE", "Request was made for non-existing error page $name");
                            return false;
                    }

                    if(!($text = apc_fetch("template_".$name))) {
                         if(!($file = fopen($this->pages[$name], "rb"))) {
                             // it's not a file
                             apc_store("template_".$name, $this->pages[$name], 1200);
                             return $this->pages[$name];
                         } else {
                             //file
                             apc_store("template_".$name, fread($file, 2**16), 600);
                             return fread($file, 2**16);
                         }
                    } else {
                            // exists in apc
                            return $text;
                    }
            }

            public function fill_template($template_name, $replace_arr) {
                if(!($templ = $this->retrieve_page($template_name))) {
                    return false;
                }

                foreach ($replace_arr as $temp => $response) {
                    $templ = str_replace("{{".$temp."}}", $response, $templ);
                }

                return $templ;
            }

            public function wrap_text($name, $text) {
                    // templates that can wrap will have a {{wrap_point}} in them.
                    
                    return $this->fill_template($name, array("wrap_point" => $text)); 
            }

    }
?>
